#include <iostream>
#include <chrono>

#include <string>
#include <kr2_program_api/api_v1/bundles/custom_device.h>

class HeartBeatMonitor {
	public:

        void processNewData(const int& a_last_sent_timestamp, const int& a_rcv_timestamp, const int& a_api_timestamp){

            checkOfTheReceivedTimestamp(a_last_sent_timestamp, a_rcv_timestamp);

            checkOfTheEchosDelay();

        }

        void reset(){
            last_received_timestamp_ = -1;
            dialog_showed_ = false;
            packet_drops_count_ = 0;
            num_stats_frames_rcv_ = 0;
        }

    private:

    int64_t last_received_timestamp_ = -1;
    int num_stats_frames_rcv_ = 0;
    bool dialog_showed_ = false;
    int packet_drops_count_ = 0;

    const unsigned int MAX_DELAY_ = std::chrono::nanoseconds(std::chrono::milliseconds(9)).count();

    const unsigned int MAX_LOST_PACKETS_ = 100;

    void doCBunError(std::string a_description){
        if (!dialog_showed_){
            auto dialog = kr2_program_api::OutputDialog::factory("KORD", a_description);
            dialog->show(false);
            dialog_showed_ = true;
        }
    }

    // checks if the echo timestamps matches the last sent by us
    void checkOfTheReceivedTimestamp(const int& a_last_sent_timestamp, const int& a_rcv_timestamp){
        if (a_last_sent_timestamp != -1 && (a_rcv_timestamp != a_last_sent_timestamp)){
            //lost packet error
            if (packet_drops_count_ > MAX_LOST_PACKETS_){
                doCBunError("To many packets lost!");
                packet_drops_count_ = 0;
            }
            packet_drops_count_ += 1;
        }    
    }

    // checks if the interval between last two received echos is reasonable
    void checkOfTheEchosDelay(){
        auto now_ts = std::chrono::high_resolution_clock::now();
        auto diff_ = now_ts.time_since_epoch().count() - last_received_timestamp_;
        if ((diff_ > MAX_DELAY_ * 100) || (last_received_timestamp_ == -1)){
            // it's timestamp from the previous run
            last_received_timestamp_ = now_ts.time_since_epoch().count();
            dialog_showed_ = false;
            num_stats_frames_rcv_ = 0;
            return;
        }
        if (diff_ > MAX_DELAY_ && (num_stats_frames_rcv_ > 3)){ // first interval may vary because the first stamp goes from syncRC()
            // if the delay is more than two heartbeats -> too much delay
            doCBunError("Too big network delay! The delay in nanos: " + std::to_string(diff_));
            //error
        }
        last_received_timestamp_ = now_ts.time_since_epoch().count();
        num_stats_frames_rcv_ += 1;
    }

};